#include "BinarySerializer.hpp"
#include "BinaryUtils.hpp"

#include <iostream>


template<class t_type>
void SerializePOD( const t_type& i_in, std::ostream& i_ostream)
{
	std::streamsize nTypeSize = sizeof(t_type);
	//i_ostream.write( reinterpret_cast<const char*>(&nTypeSize), 4);
	i_ostream.write( reinterpret_cast<const char*>(&i_in), nTypeSize);
}

//////////////////////////////////////////////////////////////////////////////////////
// CBinarySerializer : Implmentation
//////////////////////////////////////////////////////////////////////////////////////

PSF::CBinarySerializer::CBinarySerializer( std::ostream& i_ostream) :
	m_ostream( i_ostream),
	m_nStartPos( i_ostream.tellp())
{
	unsigned int nTypeSize = 0;
	m_ostream.write( reinterpret_cast<const char*>(&nTypeSize), 4);
}

PSF::CBinarySerializer::~CBinarySerializer()
{
	size_t nEndPos = m_ostream.tellp();
	size_t nTypeSize = nEndPos - m_nStartPos - 4;
	m_ostream.seekp( m_nStartPos, std::ios_base::beg);
	m_ostream.write( reinterpret_cast<const char*>(&nTypeSize), 4);
	m_ostream.seekp( nEndPos, std::ios_base::beg);
}

void PSF::CBinarySerializer::Serialize( const int& i_in)
{
	SerializePOD( i_in, m_ostream);
}

void PSF::CBinarySerializer::Serialize( const unsigned int& i_in)
{
	SerializePOD( i_in, m_ostream);
}

void PSF::CBinarySerializer::Serialize( const bool& i_in)
{
	SerializePOD( i_in, m_ostream);
}

void PSF::CBinarySerializer::Serialize( const char& i_in)
{
	SerializePOD( i_in, m_ostream);
}

void PSF::CBinarySerializer::Serialize( const unsigned char& i_in)
{
	SerializePOD( i_in, m_ostream);
}

void PSF::CBinarySerializer::Serialize( const short& i_in)
{
	SerializePOD( i_in, m_ostream);
}

void PSF::CBinarySerializer::Serialize( const unsigned short& i_in)
{
	SerializePOD( i_in, m_ostream);
}

void PSF::CBinarySerializer::Serialize( const long& i_in)
{
	SerializePOD( i_in, m_ostream);
}

void PSF::CBinarySerializer::Serialize( const unsigned long& i_in)
{
	SerializePOD( i_in, m_ostream);
}

void PSF::CBinarySerializer::Serialize( const float& i_in)
{
	SerializePOD( i_in, m_ostream);
}

void PSF::CBinarySerializer::Serialize( const double& i_in)
{
	SerializePOD( i_in, m_ostream);
}

void PSF::CBinarySerializer::Serialize( const wchar_t& i_in)
{
	SerializePOD( i_in, m_ostream);
}

std::auto_ptr<PSF::ISerializer> PSF::CBinarySerializer::SerializeCompound( const std::wstring& i_wstrName, 
																	  const Loki::TypeInfo& i_type)
{
	unsigned int nNameHash = HashString( i_wstrName);
	m_ostream.write( reinterpret_cast<const char*>(&nNameHash), 4);
	return std::auto_ptr<ISerializer>( new CBinarySerializer( m_ostream));
}

std::auto_ptr<PSF::ISerializer> PSF::CBinarySerializer::SerializeRange( const std::wstring& i_wstrName, 
																   const Loki::TypeInfo& i_type)
{
	return SerializeCompound( i_wstrName, i_type);
}


