#ifndef _DOM_SAVER_H_
#define _DOM_SAVER_H_

#include "DOM/Node.h"
#include <iostream>
#include "StringUtil.hpp"
//std::ostream& operator<<( std::ostream& i_ostream, const DOM::Node<StringT>& i_toWrite);
template<class StringT>
std::ostream& operator<<( std::ostream& i_ostream, const DOM::Node<StringT>& i_toWrite)
{
    switch ( i_toWrite.getNodeType())
    {
		case DOM::Node<StringT>::TEXT_NODE:
        {
			i_ostream << i_toWrite.getNodeValue();
            break;
        }

		case DOM::Node<StringT>::PROCESSING_INSTRUCTION_NODE :
        {
			i_ostream << "<?"  << i_toWrite.getNodeName() << ' ' << i_toWrite.getNodeValue() << "?>";
            break;
        }


		case DOM::Node<StringT>::DOCUMENT_NODE :
        {

			//std::locale loc( std::locale(), new utf16utf8_codecvt);
			//i_ostream.imbue(loc);

			i_ostream << "<?xml version=\"1.0\" encoding=\"utf-8\"?>" << std::endl;
            for ( DOM::Node<StringT> child = i_toWrite.getFirstChild(); child != 0; child = child.getNextSibling())
            {
                i_ostream << child;
                // add linefeed in requested output encoding
				i_ostream << std::endl;
            }
            break;
        }


		case DOM::Node<StringT>::ELEMENT_NODE :
        {
            // The name has to be representable without any escapes
            i_ostream << '<' << i_toWrite.getNodeName();

            // Output the element start tag.

            // Output any attributes on this element
			DOM::NamedNodeMap<StringT> attributes = i_toWrite.getAttributes();
            for ( unsigned int i = 0; i < attributes.getLength(); i++)
            {
				DOM::Node<StringT> attribute = attributes.item(i);
				i_ostream  << ' ' << attribute.getNodeName() << "=\"" << attribute.getNodeValue() << '\"';
            }

            //
            //  Test for the presence of children, which includes both
            //  text content and nested elements.
            //
			DOM::Node<StringT> child = i_toWrite.getFirstChild();
            if (child != 0)
            {
                // There are children. Close start-tag, and output children.
                // No escapes are legal here
				i_ostream << '>';

                for (; child != 0; child = child.getNextSibling())
                {
                    i_ostream << child;
                }

                //
                // Done with children.  Output the end tag.
                //
				i_ostream << "</" << i_toWrite.getNodeName() << '>';
            }
            else
            {
                //
                //  There were no children. Output the short form close of
                //  the element start tag, making it an empty-element tag.
                //
				i_ostream << "/>";
            }
            break;
        }


		case DOM::Node<StringT>::ENTITY_REFERENCE_NODE:
        {
			DOM::Node<StringT> child;
            for ( child = i_toWrite.getFirstChild(); child != 0; child = child.getNextSibling())
            {
                i_ostream << child;
            }
            break;
        }


		case DOM::Node<StringT>::CDATA_SECTION_NODE:
        {
            i_ostream << "<![CDATA[" << i_toWrite.getNodeValue() << "]]>";
            break;
        }


		case DOM::Node<StringT>::COMMENT_NODE:
        {
			i_ostream << "<!--" << i_toWrite.getNodeValue() << "-->";
            break;
        }

		/*
		case DOM::Node<StringT>::DOCUMENT_TYPE_NODE:
        {
			const DOM::DocumentType<StringT> doctype( static_cast<DOM::DocumentType<StringT> >(i_toWrite));

			i_ostream << "<!DOCTYPE " << doctype.getNodeName() << " PUBLIC \"" << 
				doctype.getPublicId() << "\" SYSTEM \"" <<
				doctype.getSystemId() << '[' << doctype.getInternalSubset() << "]>";

			break;
        }
		*/

		case DOM::Node<StringT>::ENTITY_NODE:
        {
            i_ostream << "<!ENTITY " << i_toWrite.getNodeName();

			const DOM::Entity<StringT> entity( static_cast<DOM::Entity<StringT> >(i_toWrite));
            i_ostream << "PUBLIC \"" << entity.getPublicId() << '\"';
			i_ostream << "SYSTEM \"" << entity.getSystemId() << '\"';
			i_ostream << "NDATA \"" << entity.getNotationName() << '\"';

			i_ostream << '>' << std::endl;

            break;
        }



        default:
			std::cerr << "Unrecognized node type = " << (long)i_toWrite.getNodeType() << std::endl;
    }
    return i_ostream;
}

//// ---------------------------------------------------------------------------
////  Local const data
////
////  Note: This is the 'safe' way to do these strings. If you compiler supports
////        L"" style strings, and portability is not a concern, you can use
////        those types constants directly.
//// ---------------------------------------------------------------------------
//static const XMLCh  gEndElement[] = { chOpenAngle, chForwardSlash, chNull };
//static const XMLCh  gEndPI[] = { chQuestion, chCloseAngle, chNull};
//static const XMLCh  gStartPI[] = { chOpenAngle, chQuestion, chNull };
//static const XMLCh  gXMLDecl1[] =
//{
//        chOpenAngle, chQuestion, chLatin_x, chLatin_m, chLatin_l
//    ,   chSpace, chLatin_v, chLatin_e, chLatin_r, chLatin_s, chLatin_i
//    ,   chLatin_o, chLatin_n, chEqual, chDoubleQuote, chDigit_1
//    ,   chPeriod, chDigit_0, chNull
//};
//static const XMLCh  gXMLDecl2[] =
//{
//        chDoubleQuote, chSpace, chLatin_e, chLatin_n, chLatin_c
//    ,   chLatin_o, chLatin_d, chLatin_i, chLatin_n, chLatin_g, chEqual
//    ,   chDoubleQuote, chNull
//};
//static const XMLCh  gXMLDecl3[] =
//{
//        chDoubleQuote, chQuestion, chCloseAngle
//    ,   chLF, chNull
//};
//
//static const XMLCh  gStartCDATA[] =
//{
//        chOpenAngle, chBang, chOpenSquare, chLatin_C, chLatin_D,
//        chLatin_A, chLatin_T, chLatin_A, chOpenSquare, chNull
//};
//
//static const XMLCh  gEndCDATA[] =
//{
//    chCloseSquare, chCloseSquare, chCloseAngle, chNull
//};
//static const XMLCh  gStartComment[] =
//{
//    chOpenAngle, chBang, chDash, chDash, chNull
//};
//
//static const XMLCh  gEndComment[] =
//{
//    chDash, chDash, chCloseAngle, chNull
//};
//
//static const XMLCh  gStartDoctype[] =
//{
//    chOpenAngle, chBang, chLatin_D, chLatin_O, chLatin_C, chLatin_T,
//    chLatin_Y, chLatin_P, chLatin_E, chSpace, chNull
//};
//static const XMLCh  gPublic[] =
//{
//    chLatin_P, chLatin_U, chLatin_B, chLatin_L, chLatin_I,
//    chLatin_C, chSpace, chDoubleQuote, chNull
//};
//static const XMLCh  gSystem[] =
//{
//    chLatin_S, chLatin_Y, chLatin_S, chLatin_T, chLatin_E,
//    chLatin_M, chSpace, chDoubleQuote, chNull
//};
//static const XMLCh  gStartEntity[] =
//{
//    chOpenAngle, chBang, chLatin_E, chLatin_N, chLatin_T, chLatin_I,
//    chLatin_T, chLatin_Y, chSpace, chNull
//};
//static const XMLCh  gNotation[] =
//{
//    chLatin_N, chLatin_D, chLatin_A, chLatin_T, chLatin_A,
//    chSpace, chDoubleQuote, chNull
//};

#endif