#ifndef DESERIALIZER_HPP
#define DESERIALIZER_HPP

#include <string>
#include <memory>
#include "Loki/TypeInfo.h"

namespace PSF
{
	class IDeSerializer
	{
	public:
		virtual ~IDeSerializer() = 0 {};

		virtual bool DeSerialize( char& o_out) = 0;
		virtual bool DeSerialize( unsigned char& o_out) = 0;
		virtual bool DeSerialize( short& o_out) = 0;
		virtual bool DeSerialize( unsigned short& o_out) = 0;
		virtual bool DeSerialize( int& o_out) = 0;
		virtual bool DeSerialize( unsigned int& o_out) = 0;
		virtual bool DeSerialize( long& o_out) = 0;
		virtual bool DeSerialize( unsigned long& o_out) = 0;
		virtual bool DeSerialize( float& o_out) = 0;
		virtual bool DeSerialize( double& o_out) = 0;
		// same as double
		// virtual bool DeSerialize( long double& o_out) = 0;
		virtual bool DeSerialize( bool& o_out) = 0;
		virtual bool DeSerialize( wchar_t& o_out) = 0;

		// creates a scope within the deserializer
		virtual std::auto_ptr<IDeSerializer> DeSerializeCompound( const std::wstring& i_wstrName, const Loki::TypeInfo& i_type) = 0;

		// DeSerializing a range means that all item deserialized in this scope must be of the same type
		virtual std::auto_ptr<IDeSerializer> DeSerializeRange( const std::wstring& i_wstrName, const Loki::TypeInfo& i_type) = 0;
	};
};

#endif