// Render test.cpp : Defines the entry point for the application.
//


//#include "Render test.h"
#include <iostream>
#include <fstream>
#include <list>
#include <map>
#include "Ghost/System/StreamRedirect.h"

#include "XMLSerializer.hpp"
#include "XMLDeSerializer.hpp"
#include "BinarySerializer.hpp"
#include "BinaryDeSerializer.hpp"
#include "PrettyPrintSerializer.hpp"

#include "TestBase.hpp"
#include "Serialize.hpp"


#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
// Windows Header Files:
#include <windows.h>



//////////////////////////////////////////////////////////////////////////
// Initialize data and output as xml to 'out.xml'
//////////////////////////////////////////////////////////////////////////
SerializerTypeMap CreateTypeMap()
{
	SerializerTypeMap mapType;
	mapType[typeid(int)] = L"int";
	mapType[typeid(unsigned int)] = L"unsigned int";
	mapType[typeid(float)] = L"float";
	mapType[typeid(bool)] = L"bool";
	mapType[typeid(char)] = L"char";
	mapType[typeid(wchar_t)] = L"wide char";
	mapType[typeid(std::basic_string<void>)] = L"string";
	mapType[typeid(std::list<void>)] = L"list";
	mapType[typeid(std::vector<void>)] = L"vector";
	mapType[typeid(std::map<void,void>)] = L"map";
	mapType[typeid(std::set<void>)] = L"set";
	mapType[typeid(std::pair<void,void>)] = L"pair";
	mapType[typeid(CSimple)] = L"simple";
	mapType[typeid(CComplex)] = L"complex";
	mapType[typeid(CSuper)] = L"super";
	return mapType;
}

SerializerTypeMap CreateIncompleteTypeMap()
{
	SerializerTypeMap mapType;
	mapType[typeid(int)] = L"int";
	mapType[typeid(unsigned int)] = L"unsigned int";
	//mapType[typeid(float)] = L"float";
	mapType[typeid(bool)] = L"bool";
	mapType[typeid(char)] = L"char";
	mapType[typeid(wchar_t)] = L"wide char";
	mapType[typeid(std::basic_string<void>)] = L"string";
	mapType[typeid(std::list<void>)] = L"list";
	mapType[typeid(std::vector<void>)] = L"vector";
	mapType[typeid(std::map<void,void>)] = L"map";
	mapType[typeid(std::set<void>)] = L"set";
	mapType[typeid(std::pair<void,void>)] = L"pair";
	mapType[typeid(CSimple)] = L"simple";
	mapType[typeid(CComplex)] = L"complex";
	mapType[typeid(CSuper)] = L"super";
	return mapType;
}

//////////////////////////////////////////////////////////////////////////
// Initialize data and output as xml
//////////////////////////////////////////////////////////////////////////
void Test_ToXML( std::wstring i_wstrFilenameXML)
{
	CTestBase test;
	test.DefaultInit();

	SerializerTypeMap mapType = CreateTypeMap();

	// Construct XML serializer
	PSF::CXMLSerializerProxy serializer( i_wstrFilenameXML, L"root", mapType);
	test.DoSerialize( serializer);
}

//////////////////////////////////////////////////////////////////////////
// Initialize data and output as binary
//////////////////////////////////////////////////////////////////////////
void Test_ToBinary( std::ostream& i_ostream)
{
	CTestBase test;
	test.DefaultInit();

	SerializerTypeMap mapType = CreateTypeMap();

	// Construct XML serializer
	PSF::CBinarySerializer serializer( i_ostream);
	test.DoSerialize( serializer);
}

//////////////////////////////////////////////////////////////////////////
// Deserialize data from xml file and output pretty printed to stream
//////////////////////////////////////////////////////////////////////////
void Test_XMLToPretty( std::wstring i_wstrFilenameXML, std::ostream& i_ostream)
{
	CTestBase test;

	SerializerTypeMap mapType = CreateTypeMap();

	// Construct XML Serializer
	PSF::CCheckerAssert checker;
	PSF::CXMLDeSerializerProxy deserializer( i_wstrFilenameXML, mapType, checker);

	test.DoDeSerialize( deserializer);

	// Construct pretty print serializer
	//std::wfstream fileTxt( "out.txt", std::ios::out);
	PSF::TPrettyPrintSerializer<std::ostream::char_type> serializer( i_ostream);

	test.DoSerialize( serializer);
}

//////////////////////////////////////////////////////////////////////////
// Deserialize data from xml file inverted and output pretty printed to stream
//////////////////////////////////////////////////////////////////////////
void Test_InverseXMLToPretty( std::wstring i_wstrFilenameXML, std::ostream& i_ostream)
{
	CTestBase test;

	SerializerTypeMap mapType = CreateTypeMap();

	// Construct XML Serializer
	PSF::CCheckerAssert checker;
	PSF::CXMLDeSerializerProxy deserializer( i_wstrFilenameXML, mapType, checker);

	test.DoInverseDeSerialize( deserializer);

	// Construct pretty print serializer
	//std::wfstream fileTxt( "out.txt", std::ios::out);
	PSF::TPrettyPrintSerializer<std::ostream::char_type> serializer( i_ostream);

	test.DoSerialize( serializer);
}

//////////////////////////////////////////////////////////////////////////
// Deserialize data from binary stream and output pretty printed to stream
//////////////////////////////////////////////////////////////////////////
void Test_BinaryToPretty( std::istream& i_istream, std::ostream& i_ostream)
{
	CTestBase test;

	SerializerTypeMap mapType = CreateTypeMap();

	// Construct Binary Serializer
	PSF::CBinaryDeSerializer deserializer( i_istream);

	test.DoDeSerialize( deserializer);

	// Construct pretty print serializer
	PSF::TPrettyPrintSerializer<std::ostream::char_type> serializer( i_ostream);

	test.DoSerialize( serializer);
}

//////////////////////////////////////////////////////////////////////////
// Deserialize data from binary stream inverted and output pretty printed to stream
//////////////////////////////////////////////////////////////////////////
void Test_InverseBinaryToPretty( std::istream& i_istream, std::ostream& i_ostream)
{
	CTestBase test;

	SerializerTypeMap mapType = CreateTypeMap();

	// Construct Binary Serializer
	PSF::CBinaryDeSerializer deserializer( i_istream);

	test.DoInverseDeSerialize( deserializer);

	// Construct pretty print serializer
	PSF::TPrettyPrintSerializer<std::ostream::char_type> serializer( i_ostream);

	test.DoSerialize( serializer);
}

//////////////////////////////////////////////////////////////////////////
// Deserialize data from xml files with errors to test xml deserializers
//////////////////////////////////////////////////////////////////////////
void Test_XMLError( std::ostream& i_ostream)
{
	CTestBase test;

	PSF::CCheckerException checker;
	SerializerTypeMap mapType = CreateTypeMap();
	SerializerTypeMap mapTypeIncomplete = CreateIncompleteTypeMap();

	// Testing TypeMissing
	try
	{
		PSF::CXMLDeSerializerProxy deserializer( L"MissingType.xml", mapType, checker);
		test.DoDeSerialize( deserializer);
	}
	catch ( std::runtime_error& ex)
	{
		i_ostream << "An exception occurred while deserializing MissingType.xml" << std::endl;
		i_ostream << "Expected error message : Type missing" << std::endl;
		i_ostream << "Actual error message   : " << ex.what() << std::endl;
	}

	// Testing TypeMismatch
	try
	{
		PSF::CXMLDeSerializerProxy deserializer( L"WrongType.xml", mapType, checker);
		test.DoDeSerialize( deserializer);
	}
	catch ( std::runtime_error& ex)
	{
		i_ostream << "An exception occurred while deserializing WrongType.xml" << std::endl;
		i_ostream << "Expected error message : Type mismatch" << std::endl;
		i_ostream << "Actual error message   : " << ex.what() << std::endl;
	}

	// Testing TypeUnknown
	try
	{
		PSF::CXMLDeSerializerProxy deserializer( L"out.xml", mapTypeIncomplete, checker);
		test.DoDeSerialize( deserializer);
	}
	catch ( std::runtime_error& ex)
	{
		i_ostream << "An exception occurred while deserializing out.xml" << std::endl;
		i_ostream << "Expected error message : Type unknown" << std::endl;
		i_ostream << "Actual error message   : " << ex.what() << std::endl;
	}

	// Testing SemanticError
	try
	{
		PSF::CXMLDeSerializerProxy deserializer( L"SemanticError.xml", mapType, checker);
		test.DoDeSerialize( deserializer);
	}
	catch ( std::runtime_error& ex)
	{
		i_ostream << "An exception occurred while deserializing SemanticError.xml" << std::endl;
		i_ostream << "Expected error message : Sematic error" << std::endl;
		i_ostream << "Actual error message   : " << ex.what() << std::endl;
	}

	// Testing RangeSizeError
	try
	{
		PSF::CXMLDeSerializerProxy deserializer( L"RangeError.xml", mapType, checker);
		test.DoDeSerialize( deserializer);
	}
	catch ( std::runtime_error& ex)
	{
		i_ostream << "An exception occurred while deserializing RangeError.xml" << std::endl;
		i_ostream << "Expected error message : Range size error" << std::endl;
		i_ostream << "Actual error message   : " << ex.what() << std::endl;
	}
}

void Test()
{
	// Create XML file
	std::clog << "Creating xml file" << std::endl;
	Test_ToXML( L"out.xml");

	// Create Binary file
	{
		std::clog << "Creating binary file" << std::endl;
		std::fstream fileBin( "out.bin", std::ios::out | std::ios::binary);
		Test_ToBinary( fileBin);
	}

	// Deserialize XML and binary and pretty print
	std::clog << std::endl << "Outputting deserialized xml data as pretty printed text" << std::endl;
	Test_XMLToPretty( L"out.xml", std::clog);
	std::clog << std::endl;

	std::clog << std::endl << "Outputting deserialized binary data as pretty printed text" << std::endl;
	{
		std::fstream fileBin( "out.bin", std::ios::in | std::ios::binary);
		Test_BinaryToPretty( fileBin, std::clog);
	}
	std::clog << std::endl;

	// Deserialize XML and binary inverted and pretty print
	std::clog << std::endl << "Outputting inverse deserialized xml data as pretty printed text" << std::endl;
	Test_InverseXMLToPretty( L"out.xml", std::clog);
	std::clog << std::endl;

	std::clog << std::endl << "Outputting inverse deserialized binary data as pretty printed text" << std::endl;
	{
		std::fstream fileBin( "out.bin", std::ios::in | std::ios::binary);
		Test_InverseBinaryToPretty( fileBin, std::clog);
	}
	std::clog << std::endl;

	// Testing various type errors
	Test_XMLError( std::clog);
}


int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	std::ofstream streamError( "Error.txt");
	System::TStreamRedirect<std::ios::char_type,std::ios::traits_type> streamRedirect_cerr( std::cerr, streamError);

	std::ofstream streamLog( "Log.txt");
	System::TStreamRedirect<std::ios::char_type,std::ios::traits_type> streamRedirect_clog( std::clog, streamLog);

	#if !defined(_DEBUG)
	try
	#endif
	{
		Test();

		
	}
	#if !defined(_DEBUG)
	catch( std::exception& ex)
	{
		std::cerr << "Std exception : " << ex.what() << std::endl;
	}
	catch(...)
	{
		std::cerr << "Unhandled exception" << std::endl;
	}
	#endif
	streamError.close();
	streamLog.close();

	return 1;
}
