#ifndef SERIALIZER_HPP
#define SERIALIZER_HPP

#include <string>
#include <memory>
#include "Loki/TypeInfo.h"

namespace PSF
{
	class ISerializer
	{
	public:
		virtual ~ISerializer() = 0 {};

		virtual void Serialize( const char& i_in) = 0;
		virtual void Serialize( const unsigned char& i_in) = 0;
		virtual void Serialize( const short& i_in) = 0;
		virtual void Serialize( const unsigned short& i_in) = 0;
		virtual void Serialize( const int& i_in) = 0;
		virtual void Serialize( const unsigned int& i_in) = 0;
		virtual void Serialize( const long& i_in) = 0;
		virtual void Serialize( const unsigned long& i_in) = 0;
		virtual void Serialize( const float& i_in) = 0;
		virtual void Serialize( const double& i_in) = 0;
		// same as double
		// virtual void Serialize( const long double& i_in) = 0;
		virtual void Serialize( const bool& i_in) = 0;
		virtual void Serialize( const wchar_t& i_in) = 0;

		// creates a scope within the serializer
		virtual std::auto_ptr<ISerializer> SerializeCompound( const std::wstring& i_wstrName, const Loki::TypeInfo& i_type) = 0;

		// Serializing a range means that all item serialized in this scope must be of the same type
		virtual std::auto_ptr<ISerializer> SerializeRange( const std::wstring& i_wstrName, const Loki::TypeInfo& i_type) = 0;
	};
};
#endif