#include "TestBase.hpp"
#include "Serialize.hpp"
#include "DeSerialize.hpp"

const wchar_t* CSimple::ms_namelist[] = { L"Int", L"Bool", L"Float", 0};
const wchar_t* CComplex::ms_namelist[] = { L"Simple", L"List", 0};

const wchar_t* CBase::ms_namelist[] = { L"BaseMember1", L"BaseMember2", L"BaseMember3", 0};
const wchar_t* CSuper::ms_namelist[] = { L"Member1", L"Member2", 0};

void CTestBase::DefaultInit()
{
	// POD like types
	m_wstr = L"Text";
	m_nInt = -15;
	m_nUnsignedInt = 42;
	m_fFloat = 3.14;
	m_bBool = true;

	// Simple class 
	m_simple.SetInt( 84);
	m_simple.SetBool( false);
	m_simple.SetFloat( 2.3025f);

	// Complex class 
	m_complex.Simple() = m_simple;
	m_complex.List().insert( m_complex.List().end(), 3, m_simple);

	// Super class
	m_super.BaseMember1() = 16;
	m_super.BaseMember2() = L"Base Text";
	m_super.BaseMember3() = 32;
	m_super.Member1() = m_simple;
	m_super.Member2() = 256;

	// standard containers
	int nInts[] = {1, 2, 3, 4, 5, 6, 7};
	m_list.insert( m_list.end(), nInts, nInts + 7);
	m_vector.insert( m_vector.end(), nInts, nInts + 7);
	m_set.insert( nInts, nInts + 7);
	m_map[1] = 1;
	m_map[2] = 2;
	m_map[3] = 3;
	m_map[4] = 4;
	m_map[5] = 5;
}

void CTestBase::DoSerialize( PSF::ISerializer& i_serializer)
{
	// Serialize POD types
	PSF::Serialize( m_wstr, i_serializer, L"string");
	PSF::Serialize( m_nInt, i_serializer, L"int");
	PSF::Serialize( m_nUnsignedInt, i_serializer, L"unsigned_int");
	PSF::Serialize( m_fFloat, i_serializer, L"float");
	PSF::Serialize( m_bBool, i_serializer, L"bool");

	// serialize simple class
	PSF::Serialize( m_simple, i_serializer, L"Simple");

	// serialize complex class
	PSF::Serialize( m_complex, i_serializer, L"Complex");

	// serialize super class
	PSF::Serialize( m_super, i_serializer, L"Super");

	// serialize list
	PSF::Serialize( m_list, i_serializer, L"number_list");

	// serialize vector
	PSF::Serialize( m_vector, i_serializer, L"number_vector");

	// serialize map
	PSF::Serialize( m_map, i_serializer, L"number_map");

	// serialize set
	PSF::Serialize( m_set, i_serializer, L"number_set");
}

void CTestBase::DoDeSerialize( PSF::IDeSerializer& i_deserializer)
{
	// Deserialize POD types
	PSF::DeSerialize( m_wstr, i_deserializer, L"string");
	PSF::DeSerialize( m_nInt, i_deserializer, L"int");
	PSF::DeSerialize( m_nUnsignedInt, i_deserializer, L"unsigned_int");
	PSF::DeSerialize( m_fFloat, i_deserializer, L"float");
	PSF::DeSerialize( m_bBool, i_deserializer, L"bool");

	// Deserialize simple class
	PSF::DeSerialize( m_simple, i_deserializer, L"Simple");

	// Deserialize complex class
	PSF::DeSerialize( m_complex, i_deserializer, L"Complex");

	// Deserialize super class
	PSF::DeSerialize( m_super, i_deserializer, L"Super");

	// Deserialize list
	PSF::DeSerialize( m_list, i_deserializer, L"number_list");

	// Deserialize vector
	PSF::DeSerialize( m_vector, i_deserializer, L"number_vector");

	// Deserialize map
	PSF::DeSerialize( m_map, i_deserializer, L"number_map");

	// Deserialize set
	PSF::DeSerialize( m_set, i_deserializer, L"number_set");
}

void CTestBase::DoInverseDeSerialize( PSF::IDeSerializer& i_deserializer)
{
	// Deserialize set
	PSF::DeSerialize( m_set, i_deserializer, L"number_set");

	// Deserialize map
	PSF::DeSerialize( m_map, i_deserializer, L"number_map");

	// Deserialize vector
	PSF::DeSerialize( m_vector, i_deserializer, L"number_vector");

	// Deserialize list
	PSF::DeSerialize( m_list, i_deserializer, L"number_list");

	// Deserialize super class
	PSF::DeSerialize( m_super, i_deserializer, L"Super");

	// Deserialize complex class
	PSF::DeSerialize( m_complex, i_deserializer, L"Complex");

	// Deserialize simple class
	PSF::DeSerialize( m_simple, i_deserializer, L"Simple");

	// Deserialize POD types
	PSF::DeSerialize( m_bBool, i_deserializer, L"bool");
	PSF::DeSerialize( m_fFloat, i_deserializer, L"float");
	PSF::DeSerialize( m_nUnsignedInt, i_deserializer, L"unsigned_int");
	PSF::DeSerialize( m_nInt, i_deserializer, L"int");
	PSF::DeSerialize( m_wstr, i_deserializer, L"string");
}
