#ifndef XML_DESERIALIZER_HPP
#define XML_DESERIALIZER_HPP

#include "DeSerializer.hpp"
#include "TypeMap.hpp"
#include "Checker.hpp"

#include <xercesc/dom/DOM.hpp>

#include <sstream>

namespace PSF
{
	class CXMLDeSerializerBase : public IDeSerializer
	{
	public:
		CXMLDeSerializerBase( XERCES_CPP_NAMESPACE::DOMElement* i_pdomElementRoot, 
							  bool i_bSearch,
							  const SerializerTypeMap& i_mapType,
							  const IChecker& i_checker);

		virtual ~CXMLDeSerializerBase();

		virtual bool DeSerialize( char& o_out);
		virtual bool DeSerialize( unsigned char& o_out);
		virtual bool DeSerialize( short& o_out);
		virtual bool DeSerialize( unsigned short& o_out);
		virtual bool DeSerialize( int& o_out);
		virtual bool DeSerialize( unsigned int& o_out);
		virtual bool DeSerialize( long& o_out);
		virtual bool DeSerialize( unsigned long& o_out);
		virtual bool DeSerialize( float& o_out);
		virtual bool DeSerialize( double& o_out);
		// same as double
		// virtual bool DeSerialize( long double& o_out);
		virtual bool DeSerialize( wchar_t& o_out);
		virtual bool DeSerialize( bool& o_out);

		virtual std::auto_ptr<IDeSerializer> DeSerializeCompound( const std::wstring& i_wstrName, 
																  const Loki::TypeInfo& i_type);
		virtual std::auto_ptr<IDeSerializer> DeSerializeRange( const std::wstring& i_wstrName, 
															   const Loki::TypeInfo& i_type);
	protected:
		// returns the element used for deserialization and updates the current element
		XERCES_CPP_NAMESPACE::DOMElement*  UpdateCurrentElement( const std::wstring& i_wstrName);
		void CheckType( const std::wstring& i_wstrType, const Loki::TypeInfo& i_type);

		CXMLDeSerializerBase();
		XERCES_CPP_NAMESPACE::DOMElement* m_pdomElementRoot;
		XERCES_CPP_NAMESPACE::DOMElement* m_pdomElementCurrent;
		bool m_bSearch;
		const SerializerTypeMap& m_mapType;
		const IChecker& m_checker;
		size_t m_nItemsDeSerialized;
		size_t m_nRangeSize;

		std::wstringstream m_wstrstream;
	};

	class CXMLDeSerializerProxy
	{
	public:
		CXMLDeSerializerProxy( const std::wstring& i_wstrFilename,
						  const SerializerTypeMap& i_mapType,
						  const IChecker& i_checker);

		virtual ~CXMLDeSerializerProxy();

		operator IDeSerializer&();

	protected:
		XERCES_CPP_NAMESPACE::DOMImplementation* m_pdomImpl;
		XERCES_CPP_NAMESPACE::DOMBuilder* m_pdomBuilder;
		XERCES_CPP_NAMESPACE::DOMDocument* m_pdomDoc;
		std::auto_ptr<CXMLDeSerializerBase> m_pBase;
	};
};

#endif