#ifndef XML_SERIALIZER_HPP
#define XML_SERIALIZER_HPP

#include "Serializer.hpp"
#include "TypeMap.hpp"

#include <xercesc/dom/DOM.hpp>

#include <sstream>

namespace PSF
{
	class CXMLSerializerBase : public ISerializer
	{
	public:
		CXMLSerializerBase( XERCES_CPP_NAMESPACE::DOMDocument* i_pdomDoc, 
							XERCES_CPP_NAMESPACE::DOMElement* i_pdomElementRoot, 
							const SerializerTypeMap& i_mapType,
							bool i_bRange);

		virtual ~CXMLSerializerBase();
		virtual void Serialize( const int& i_in);
		virtual void Serialize( const unsigned int& i_in);
		virtual void Serialize( const bool& i_in);
		virtual void Serialize( const char& i_in);
		virtual void Serialize( const unsigned char& i_in);
		virtual void Serialize( const short& i_in);
		virtual void Serialize( const unsigned short& i_in);
		virtual void Serialize( const long& i_in);
		virtual void Serialize( const unsigned long& i_in);
		virtual void Serialize( const float& i_in);
		virtual void Serialize( const double& i_in);
		// same as double
		// virtual void Serialize( const long double& i_in);
		virtual void Serialize( const wchar_t& i_in);

		virtual std::auto_ptr<ISerializer> SerializeCompound( const std::wstring& i_wstrName, 
															  const Loki::TypeInfo& i_type);
		virtual std::auto_ptr<ISerializer> SerializeRange( const std::wstring& i_wstrName, 
														   const Loki::TypeInfo& i_type);

	protected:
		CXMLSerializerBase();
		XERCES_CPP_NAMESPACE::DOMDocument* m_pdomDoc;
		XERCES_CPP_NAMESPACE::DOMElement* m_pdomElementRoot;
		const SerializerTypeMap& m_mapType;
		size_t m_nItemsSerialized;
		bool m_bRange;
		std::wstringstream m_wstrstream;
	};

	class CXMLSerializerProxy
	{
	public:
		CXMLSerializerProxy( const std::wstring& i_wstrFilename, 
						const std::wstring& i_wstrRoot, 
						const SerializerTypeMap& i_mapType = SerializerTypeMap());

		virtual ~CXMLSerializerProxy();

		operator ISerializer&();

	protected:
		std::wstring m_wstrFilename;
		XERCES_CPP_NAMESPACE::DOMImplementation* m_pdomImpl;
		XERCES_CPP_NAMESPACE::DOMDocument* m_pdomDoc;
		std::auto_ptr<CXMLSerializerBase> m_pBase;
	};
};

#endif